@@ -18,9 +18,12 @@ from pyomo.contrib.gdpopt.util import copy_var_list_values, SuppressInfeasibleWa
 from pyomo.contrib.gdpopt.mip_solve import distinguish_mip_infeasible_or_unbounded
 from pyomo.solvers.plugins.solvers.persistent_solver import PersistentSolver
 from pyomo.common.dependencies import attempt_import
-from pyomo.contrib.mindtpy.util import generate_norm1_objective_function, generate_norm2sq_objective_function, generate_norm_inf_objective_function, generate_lag_objective_function, set_solver_options
+from pyomo.contrib.mindtpy.util import generate_norm1_objective_function, generate_norm2sq_objective_function, generate_norm_inf_objective_function, generate_lag_objective_function, set_solver_options, GurobiPersistent4MindtPy
+
+
 logger = logging.getLogger('pyomo.contrib.mindtpy')
 
+
 single_tree, single_tree_available = attempt_import(
     'pyomo.contrib.mindtpy.single_tree')
 tabu_list, tabu_list_available = attempt_import(
@@ -85,6 +88,9 @@ def solve_main(solve_data, config, fp=False, regularization_problem=False):
                                       "No-good cuts are added and GOA algorithm doesn't converge within the time limit. "
                                       'No integer solution is found, so the cplex solver will report an error status. ')
         return None, None
+    if config.solution_pool:
+        main_mip_results._solver_model = mainopt._solver_model
+        main_mip_results._pyomo_var_to_solver_var_map = mainopt._pyomo_var_to_solver_var_map
     if main_mip_results.solver.termination_condition is tc.optimal:
         if config.single_tree and not config.add_no_good_cuts and not regularization_problem:
             if solve_data.objective_sense == minimize:
@@ -127,22 +133,31 @@ def setup_mip_solver(solve_data, config, regularization_problem):
     if regularization_problem:
         mainopt = SolverFactory(config.mip_regularization_solver)
     else:
-        mainopt = SolverFactory(config.mip_solver)
+        if config.mip_solver == 'gurobi_persistent' and config.single_tree:
+            mainopt = GurobiPersistent4MindtPy()
+            mainopt.solve_data = solve_data
+            mainopt.config = config
+        else:
+            mainopt = SolverFactory(config.mip_solver)
+
     # determine if persistent solver is called.
     if isinstance(mainopt, PersistentSolver):
         mainopt.set_instance(solve_data.mip, symbolic_solver_labels=True)
     if config.single_tree and not regularization_problem:
-        # Configuration of lazy callback
-        lazyoa = mainopt._solver_model.register_callback(
-            single_tree.LazyOACallback_cplex)
-        # pass necessary data and parameters to lazyoa
-        lazyoa.main_mip = solve_data.mip
-        lazyoa.solve_data = solve_data
-        lazyoa.config = config
-        lazyoa.opt = mainopt
-        mainopt._solver_model.set_warning_stream(None)
-        mainopt._solver_model.set_log_stream(None)
-        mainopt._solver_model.set_error_stream(None)
+        # Configuration of cplex lazy callback
+        if config.mip_solver == 'cplex_persistent':
+            lazyoa = mainopt._solver_model.register_callback(
+                single_tree.LazyOACallback_cplex)
+            # pass necessary data and parameters to lazyoa
+            lazyoa.main_mip = solve_data.mip
+            lazyoa.solve_data = solve_data
+            lazyoa.config = config
+            lazyoa.opt = mainopt
+            mainopt._solver_model.set_warning_stream(None)
+            mainopt._solver_model.set_log_stream(None)
+            mainopt._solver_model.set_error_stream(None)
+        if config.mip_solver == 'gurobi_persistent':
+            mainopt.set_callback(single_tree.LazyOACallback_gurobi)
     if config.use_tabu_list:
         tabulist = mainopt._solver_model.register_callback(
             tabu_list.IncumbentCallback_cplex)
