@@ -12,6 +12,8 @@ from pyomo.core.base.var import IndexedVar
 from pyomo.core.base.set_types import Reals
 from pyomo.opt import TerminationCondition as tc
 from pyomo.core.expr import value
+from pyomo.core.expr import current as EXPR
+from pyomo.core.expr.numeric_expr import NPV_MaxExpression, NPV_MinExpression
 from pyomo.repn.standard_repn import generate_standard_repn
 from pyomo.core.expr.visitor import identify_variables, identify_mutable_parameters, replace_expressions
 from pyomo.core.expr.sympy_tools import sympyify_expression, sympy2pyomo_expression
@@ -279,6 +281,80 @@ def transform_to_standard_form(model):
     return
 
 
+def get_vars_from_component(block, ctype):
+    """Determine all variables used in active components within a block.
+
+    Parameters
+    ----------
+    block: Block
+        The block to search for components.  This is a recursive
+        generator and will descend into any active sub-Blocks as well.
+    ctype:  class
+        The component type (typically either :py:class:`Constraint` or
+        :py:class:`Objective` to search for).
+
+    """
+
+    seen = set()
+    for compdata in block.component_data_objects(
+            ctype,
+            descend_into=True,
+            active=True):
+        for var in EXPR.identify_variables(compdata.expr):
+            if id(var) not in seen:
+                seen.add(id(var))
+                yield var
+
+
+def replace_uncertain_bounds_with_constraints(model, uncertain_params):
+    """
+    For variables of which the bounds are dependent on the parameters
+    in the list `uncertain_params`, remove the bounds and add
+    explicit variable bound inequality constraints.
+
+    :param model: Model in which to make the bounds/constraint replacements
+    :type model: class:`pyomo.core.base.PyomoModel.ConcreteModel`
+    :param uncertain_params: List of uncertain model parameters
+    :type uncertain_params: list
+    """
+    uncertain_param_set = ComponentSet(uncertain_params)
+
+    # component for explicit inequality constraints
+    uncertain_var_bound_constrs = ConstraintList()
+    model.add_component(unique_component_name(model,
+                                              'uncertain_var_bound_cons'),
+                        uncertain_var_bound_constrs)
+
+    # get all variables in active objective and constraint expression(s)
+    vars_in_cons = ComponentSet(get_vars_from_component(model, Constraint))
+    vars_in_obj = ComponentSet(get_vars_from_component(model, Objective))
+
+    for v in vars_in_cons | vars_in_obj:
+        # get mutable parameters in variable bounds expressions
+        ub = v.upper
+        mutable_params_ub = ComponentSet(identify_mutable_parameters(ub))
+        lb = v.lower
+        mutable_params_lb = ComponentSet(identify_mutable_parameters(lb))
+
+        # add explicit inequality constraint(s), remove variable bound(s)
+        if mutable_params_ub & uncertain_param_set:
+            if type(ub) is NPV_MinExpression:
+                upper_bounds = ub.args
+            else:
+                upper_bounds = (ub,)
+            for u_bnd in upper_bounds:
+                uncertain_var_bound_constrs.add(v - u_bnd <= 0)
+            v.setub(None)
+        if mutable_params_lb & uncertain_param_set:
+            if type(ub) is NPV_MaxExpression:
+                lower_bounds = lb.args
+            else:
+                lower_bounds = (lb,)
+            for l_bnd in lower_bounds:
+                uncertain_var_bound_constrs.add(l_bnd - v <= 0)
+            v.setlb(None)
+
+
 def validate_kwarg_inputs(model, config):
     '''
     Confirm kwarg inputs satisfy PyROS requirements.
